import { promises as fs, existsSync } from "fs";
class DataStore {
  #collections = /* @__PURE__ */ new Map();
  constructor() {
    this.#collections = /* @__PURE__ */ new Map();
  }
  get(collectionName, key) {
    return this.#collections.get(collectionName)?.get(String(key));
  }
  entries(collectionName) {
    const collection = this.#collections.get(collectionName) ?? /* @__PURE__ */ new Map();
    return collection.entries();
  }
  set(collectionName, key, value) {
    const collection = this.#collections.get(collectionName) ?? /* @__PURE__ */ new Map();
    collection.set(String(key), value);
    this.#collections.set(collectionName, collection);
  }
  delete(collectionName, key) {
    const collection = this.#collections.get(collectionName);
    if (collection) {
      collection.delete(String(key));
    }
  }
  clear(collectionName) {
    this.#collections.delete(collectionName);
  }
  has(collectionName, key) {
    const collection = this.#collections.get(collectionName);
    if (collection) {
      return collection.has(String(key));
    }
    return false;
  }
  hasCollection(collectionName) {
    return this.#collections.has(collectionName);
  }
  collections() {
    return this.#collections;
  }
  scopedStore(collectionName) {
    return {
      get: (key) => this.get(collectionName, key),
      entries: () => this.entries(collectionName),
      set: (key, value) => this.set(collectionName, key, value),
      delete: (key) => this.delete(collectionName, key),
      clear: () => this.clear(collectionName),
      has: (key) => this.has(collectionName, key)
    };
  }
  metaStore(collectionName) {
    return this.scopedStore(`meta:${collectionName}`);
  }
  toString() {
    return JSON.stringify(
      Array.from(this.#collections.entries()).map(([collectionName, collection]) => {
        return [collectionName, Array.from(collection.entries())];
      })
    );
  }
  async writeToDisk(filePath) {
    await fs.writeFile(filePath, this.toString());
  }
  static async fromDisk(filePath) {
    if (!existsSync(filePath)) {
      return new DataStore();
    }
    const str = await fs.readFile(filePath, "utf-8");
    return DataStore.fromString(str);
  }
  static fromString(str) {
    const entries = JSON.parse(str);
    return DataStore.fromJSON(entries);
  }
  static async fromModule() {
    try {
      const data = await import("astro:data-layer-content");
      return DataStore.fromJSON(data.default);
    } catch {
    }
    return new DataStore();
  }
  static fromJSON(entries) {
    const collections = /* @__PURE__ */ new Map();
    for (const [collectionName, collection] of entries) {
      collections.set(collectionName, new Map(collection));
    }
    const store = new DataStore();
    store.#collections = collections;
    return store;
  }
}
function dataStoreSingleton() {
  let instance = void 0;
  return {
    get: async () => {
      if (!instance) {
        instance = DataStore.fromModule();
      }
      return instance;
    },
    set: (store) => {
      instance = store;
    }
  };
}
const globalDataStore = dataStoreSingleton();
export {
  DataStore,
  globalDataStore
};
