import { DataStore, globalDataStore } from "./data-store.js";
import { getEntryData, globalContentConfigObserver } from "./utils.js";
import { promises as fs, existsSync } from "fs";
async function syncDataLayer({
  settings,
  logger: globalLogger,
  store
}) {
  const logger = globalLogger.forkIntegrationLogger("content");
  if (!store) {
    store = await DataStore.fromDisk(new URL("data-store.json", settings.config.cacheDir));
    globalDataStore.set(store);
  }
  const contentConfig = globalContentConfigObserver.get();
  if (contentConfig?.status !== "loaded") {
    logger.debug("Content config not loaded, skipping sync");
    return;
  }
  await Promise.all(
    Object.entries(contentConfig.config.collections).map(async ([name, collection]) => {
      if (collection.type !== "experimental_data") {
        return;
      }
      let { schema } = collection;
      if (!schema) {
        schema = collection.loader.schema;
      }
      if (typeof schema === "function") {
        schema = await schema({
          image: () => {
            throw new Error("Images are currently not supported for experimental data collections");
          }
        });
      }
      const collectionWithResolvedSchema = { ...collection, schema };
      function parseData({
        id,
        data,
        filePath = ""
      }) {
        return getEntryData(
          {
            id,
            collection: name,
            unvalidatedData: data,
            _internal: {
              rawData: void 0,
              filePath
            }
          },
          collectionWithResolvedSchema,
          false
        );
      }
      return collection.loader.load({
        collection: name,
        store: store.scopedStore(name),
        meta: store.metaStore(name),
        logger,
        settings,
        parseData
      });
    })
  );
  const cacheFile = new URL("data-store.json", settings.config.cacheDir);
  if (!existsSync(settings.config.cacheDir)) {
    await fs.mkdir(settings.config.cacheDir, { recursive: true });
  }
  await store.writeToDisk(cacheFile);
  logger.info("Synced content");
}
export {
  syncDataLayer
};
